local waitTable = {};
local waitFrame = nil;

-- saved buttons
CustomButtonsData = {};

-- Konfiguration
CustomButtonsConfig = {};

isInConfig = false;
loaded = false;

CustomButtonsConfig.ShowCount = 1;
CustomButtonsConfig.Sound = 1;
CustomButtonsConfig.RangeIndicator = 0;

local selectedButton = nil;

local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

function CustomButtons_StartDelay(delay, func, ...)
    if(type(delay)~="number" or type(func)~="function") then
        return false;
    end
    
    if(waitFrame == nil) then
        waitFrame = CreateFrame("Frame","CustomButtons_WaitFrame", UIParent);
        waitFrame:SetScript("onUpdate",function (self,elapse)
            local count = #waitTable;
            local i = 1;
            
            while(i<=count) do
                local waitRecord = tremove(waitTable,i);
                local d = tremove(waitRecord,1);
                local f = tremove(waitRecord,1);
                local p = tremove(waitRecord,1);
                
                if(d>elapse) then
                    tinsert(waitTable,i,{d-elapse,f,p});
                    i = i + 1;
                else
                    count = count - 1;
                    f(unpack(p));
                end
            end
        end);
    end
    
    tinsert(waitTable,{delay,func,{...}});
    
    return true;
end

-- functions
function CustomButtons_OnLoad(self)
 	DEFAULT_CHAT_FRAME:AddMessage("|c333399ffNirriti's |rCustomButtons loaded.", 1.0, 1.0, 1.0, 1, 10);
 
 	self:RegisterEvent("ADDON_LOADED");
 	self:RegisterEvent("MODIFIER_STATE_CHANGED");
 	self:RegisterEvent("PLAYER_TALENT_UPDATE");
 	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
 	self:RegisterEvent("LEARNED_SPELL_IN_TAB");
    self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");    
    self:RegisterEvent("PET_BATTLE_OPENING_START"); 
    self:RegisterEvent("PET_BATTLE_CLOSE"); 
 	
    hooksecurefunc("ActionButton_OnUpdate", CustomButtons_ActionButton_OnUpdate);
end

function CustomButtons_InitSlashCommandHandler()

	SLASH_CUSTOMBUTTONS1 = "/custombuttons";
  	SLASH_CUSTOMBUTTONS2 = "/cb";
  	SlashCmdList["CUSTOMBUTTONS"] = function(msg)
		CustomButtons_SlashCommandHandler(msg);
	end
end

local debug = false;
function cb_debug(value, name)
    if (not debug) then
        return;
    end

    if (name) then
 	    DEFAULT_CHAT_FRAME:AddMessage("debug "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
 	    DEFAULT_CHAT_FRAME:AddMessage("debug: "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

function CustomButtons_SlashCommandHandler(msg)
	DEFAULT_CHAT_FRAME:AddMessage("CustomButtons: "..msg, 1.0, 1.0, 1.0, 1, 10);
		
	if (msg == "reload") then
	 	ReloadUI();
	elseif (string.find(msg, "new") == 1) then
	 	CustomButtons_CreateNewButton();
	 	
		local str_scale = string.sub(msg, 4);
		local scale = tonumber(str_scale);
		
		if (scale and scale >= 0.1 and scale <= 4) then
            if (selectedButton) then
                CustomButtons_SetButtonScale(selectedButton, scale);
                CustomButtons_SaveButton(selectedButton);
            end
		end
	elseif (string.find(msg, "scale") == 1) then
		local str_scale = string.sub(msg, 7);
		local scale = tonumber(str_scale);
		
		if (scale and scale >= 0.1 and scale <= 4) then
            if (selectedButton) then
                CustomButtons_SetButtonScale(selectedButton, scale);
                CustomButtons_SaveButton(selectedButton);
            end
        else
			DEFAULT_CHAT_FRAME:AddMessage(CB_FORMAT_ERROR..": "..msg, 1.0, 0, 0, 1, 10);
	 	end
	elseif (string.find(msg, "alpha") == 1) then
		local str_alpha = string.sub(msg, 7);
		local alpha = tonumber(str_alpha);
		
		if (alpha and alpha >= 0 and alpha <= 1) then
            if (selectedButton) then
                CustomButtons_SetButtonAlpha(selectedButton, alpha);
                CustomButtons_SaveButton(selectedButton);
            end
        else
			DEFAULT_CHAT_FRAME:AddMessage(CB_FORMAT_ERROR..": "..msg, 1.0, 0, 0, 1, 10);
	 	end
	elseif (string.find(msg, "toggle") == 1) then
		local target = string.sub(msg, 8);
		
		if (target == "count") then
			CustomButtons_ToggleCount();
		elseif (target == "sound") then
			CustomButtons_ToggleSound();
		elseif (target == "range") then
			if (CustomButtonsConfig.RangeIndicator == 1) then
			    CustomButtonsConfig.RangeIndicator = 0;
			else
			    CustomButtonsConfig.RangeIndicator = 1;
			end
		elseif (target == "talent") then
		    CustomButtons_ToggleTalent();
		else
			DEFAULT_CHAT_FRAME:AddMessage(CB_FORMAT_ERROR..": "..msg, 1.0, 0, 0, 1, 10);
		end
	elseif (string.find(msg, "sound") == 1) then
		local sound = string.sub(msg, 7);
		
		if (sound == "?") then
		    CustomButtons_ShowButtonSoundInfo();
		elseif (sound == "none" or sound == "") then
		    CustomButtons_SetButtonSound(nil);
		else
		    CustomButtons_SetButtonSound(sound);
		end
	elseif (string.find(msg, "key") == 1) then
		local key = string.sub(msg, 5);
		
	    local button = selectedButton;
	
		if (key == "none" or key == "") then
		    CustomButtons_SetKeyBinding(nil, button);
		else
		    CustomButtons_SetKeyBinding(key, button);
		end
	elseif (msg == "delete") then
	 	CustomButtons_DeleteButton();
	elseif (msg == "reset") then
	 	wipe(CustomButtonsData);
    elseif (string.find(msg, "id") == 1) then
        local spellname = string.sub(msg, 4);
        
        if (spellname) then
            local id = CustomButtons_GetSpellID(spellname);
            DEFAULT_CHAT_FRAME:AddMessage(spellname..": "..(id or "nil"), 1.0, 0, 0, 1, 10);
        end
	end
	
	CustomButtons_GetSpellID(name)
end

function CustomButtons_ToggleCount()
    if (CustomButtonsConfig.ShowCount == 1) then
        CustomButtonsConfig.ShowCount = 0;
    else
        CustomButtonsConfig.ShowCount = 1;
    end
end

function CustomButtons_ToggleSound()
    if (CustomButtonsConfig.Sound == 1) then
        CustomButtonsConfig.Sound = 0;
    else
        CustomButtonsConfig.Sound = 1;
    end
end

function CustomButtons_ToggleRangeIndicator()
    if (CustomButtonsConfig.RangeIndicator == 1) then
        CustomButtonsConfig.RangeIndicator = 0;
    else
        CustomButtonsConfig.RangeIndicator = 1;
    end
end

function CustomButtons_AddonLoaded()
    if (MASQUE) then
        buttongroup = MASQUE:Group("CustomButtons");
    end

    CustomButtons_CleanupButtons();
    
    CustomButtons_InitSlashCommandHandler();
    CustomButtons_InitButtons();
    
    CustomButtons_ReinitSpells();            
    CustomButtons_ReSkin();
    
    loaded = true;
end

function CustomButtons_OnEvent(self, event, ...)
 	
	if (event == "ADDON_LOADED") then
	
		local addonName = ...;
		if (addonName == "CustomButtons") then
		    CustomButtons_StartDelay(5, CustomButtons_AddonLoaded);
		end
 	end
 	
 	if (not loaded) then
 	    return;
 	end
 	
 	if (event == "PLAYER_TALENT_UPDATE" or
 	    event == "ACTIVE_TALENT_GROUP_CHANGED" or
 	    event == "LEARNED_SPELL_IN_TAB" or
 	    event == "UPDATE_SHAPESHIFT_FORM") then
        
        if (InCombatLockdown() == nil) then
            CustomButtons_StartDelay(2, CustomButtons_ReinitSpells);
 	    end
 	end
 	
    -- pet battles
    if (event == "PET_BATTLE_OPENING_START") then
        CustomButtonsFrame:Hide();
    end
    if (event == "PET_BATTLE_CLOSE") then
        CustomButtonsFrame:Show();
    end
    
 	if (event == "PLAYER_ENTERING_WORLD") then        
        CustomButtons_ReSkin();
 	end
end

function CustomButtons_ReSkin()   
    if (MASQUE) then
        buttongroup:ReSkin();
    end

    for index, data in pairs(CustomButtonsData) do
        local button = _G[data.name];
        CustomButtons_SetTextScale(button, data.scale);
    end
end

function CustomButtons_ReinitSpells()
    local activeSpecGroup = GetActiveSpecGroup();

    if (not activeSpecGroup) then
        activeSpecGroup = 1;
    end

    for index, data in pairs(CustomButtonsData) do 
        
		local name = data.name;
        local button = _G[name];          
        local show = false;
     
        if (data) then
            if (data.type == "spell") then         
                if (data.talent == 0 or data.talent == activeSpecGroup) then                
                    button:SetAttribute("type", "spell");
                
                    local spellname = data.spell;
                    if (data.spell2 and strlen(data.spell2) > 0) then
                        spellname = data.spell.."("..data.spell2..")";
                    end
                
                    button:SetAttribute("spell", spellname);
    			
                    button:SetAttribute("cb_sp1", data.spell);
                    button:SetAttribute("cb_sp2", data.spell2);
                    
                    local id = CustomButtons_findSpellBookId(data.spell, data.spell2, data.id);
                    
                    if (id and id >= 0) then
                        button:SetAttribute("id", id);	
                        data.id = id;	    
                        local name, subname, texture = GetSpellInfo(id, SpellBookFrame.bookType);
                        
                        local icon = _G[data.name.."Icon"];
                        icon:SetTexture(texture); 
                        show = true;
                    end
                end
            elseif (data.type == "macro") then 
                if (data.talent == 0 or data.talent == activeSpecGroup) then  
                    show = true;
                end
            elseif (data.type == "item") then 
                if (data.talent == 0 or data.talent == activeSpecGroup) then  
                    show = true;
                end
            else
                show = true;
            end
        end
        
        if (show) then   
            button:Show();            
            CustomButtons_SetKeyBinding(data.key, button);     
        else
            button:Hide();
            ClearOverrideBindings(button);
        end
    end
end

function CustomButtons_OnButtonEvent(self, event, ...)
 	
	if ( event == "PLAYER_TARGET_CHANGED" ) then
		self.rangeTimer = -1;
	elseif ( event == "ACTIONBAR_UPDATE_USABLE" ) then
		CustomButtons_UpdateUsable(self);
	elseif ( event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
		CustomButtons_UpdateCooldowns(self);
 	end
end

function CustomButtons_CleanupButtons()

    local removed = nil;

	for index, data in pairs(CustomButtonsData) do		
		if (data.type == nil) then
            table.remove(CustomButtonsData, index);
            removed = 1;
            break;
		end
		
		if (data.type == "spell") then            
            if (data.talent == nil) then
		        table.remove(CustomButtonsData, index);
		        removed = 1;
		        break;
            end
		end
	end
	
	-- recursive
	if (removed == 1) then
	    CustomButtons_CleanupButtons();
	end
end

function CustomButtons_InitButtons()
	for index, data in pairs(CustomButtonsData) do
	
		local type = data.type;
		local name = data.name;
		local texture = data.texture;
		local sound = data.sound;
		local key = data.key;
		
		local p1 = data.p1;
		local p2 = data.p2;
				
		local x = data.x;
		local y = data.y;
		
		local scale = data.scale;
		if (not scale) then
			scale = 1.0;
		end
		
		local alpha = data.alpha;
		if (not alpha) then
			alpha = 1.0;
		end

		local button = CustomButtons_CreateButton(name);
		CustomButtons_ResetAttributes(button);
	
		CustomButtons_SetButtonScale(button, scale);
		CustomButtons_SetButtonAlpha(button, alpha);
		
		if (type == "item") then
			
			button:SetAttribute("type", "item");
			button:SetAttribute("item", data.item);
           
            local talent = data.talent;
            
            if (not talent) then
                data.talent = 1;
            end
			
		elseif (type == "spell") then
			
			button:SetAttribute("type", "spell");
            
            local spellname = data.spell;
            if (data.spell2 and strlen(data.spell2) > 0) then
                spellname = data.spell.."("..data.spell2..")";
            end
            
			button:SetAttribute("spell", spellname);
			
            button:SetAttribute("cb_sp1", data.spell);
            button:SetAttribute("cb_sp2", data.spell2);
			
			local id = CustomButtons_findSpellBookId(data.spell, data.spell2, data.id);
		    button:SetAttribute("id", id);
		   
            local talent = data.talent;
            
            if (not talent) then
                data.talent = 1;
            end
			
		elseif (type == "macro") then
		
			button:SetAttribute("type", "macro");
			button:SetAttribute("macro", data.macro);
			
			button:SetAttribute("tooltip", data.macro);
		   
            local talent = data.talent;
            
            if (not talent) then
                data.talent = 1;
            end
			
		elseif (type == "equipmentset") then
		
			button:SetAttribute("type", "macro");
			button:SetAttribute("macrotext", "/equipset "..data.equipmentset);
			
			button:SetAttribute("tooltip", data.equipmentset);
			
		elseif (type == "companion") then
		
			button:SetAttribute("type", "macro");
			
            if (data.ctype == "CRITTER") then
                button:SetAttribute("macrotext", "/run CallCompanion(\""..data.ctype.."\", "..data.index..")");
            else
                button:SetAttribute("macrotext", "/run if IsMounted() then Dismount() else CallCompanion(\""..data.ctype.."\", "..data.index..") end");
            end
			
			button:SetAttribute("tooltip", data.index);
			
		end
			
		local icon = _G[name.."Icon"];
		icon:SetTexture(texture);
		
		button:SetAttribute("sound", sound);
		button:SetAttribute("key", key);
		
		button:ClearAllPoints();
		button:SetPoint(p1, UIParent, p2, x, y);
		
		button:SetScript("OnMouseDown", CustomButtons_OnMouseDown);
		button:SetScript("OnMouseUp", CustomButtons_OnMouseUp);
		
		local hotkey = _G[name.."HotKey"];
		    
		if (key) then
		    SetOverrideBindingClick(button, false, key, button:GetName());
		    
            CustomButtons_SetKeyBindingText(hotkey, key);
            hotkey:Show();
        else
            ClearOverrideBindings(button);
            
            hotkey:SetText(RANGE_INDICATOR);
            hotkey:Hide();
		end
		
		if (MASQUE) then
		    buttongroup:AddButton(button);
		end
	end
end

function CustomButtons_SetKeyBindingText(hotkey, key)
    local text = GetBindingText(key, "KEY_", 1);
    hotkey:SetText(text);
end

function CustomButtons_SetButtonScale(button, scale)
    button:SetScale(scale);
    CustomButtons_SetTextScale(button, scale);
end

function CustomButtons_SetTextScale(button, scale)    

    local fontName, fontHeight, fontFlags;

    -- rescale count
    local counttext = _G[button:GetName().."Count"];
    fontName, fontHeight, fontFlags = counttext:GetFont();
    counttext:SetFont(fontName, 14/scale, fontFlags);
    
    -- rescale hotkey
    local hotkey = _G[button:GetName().."HotKey"];
    fontName, fontHeight, fontFlags = hotkey:GetFont();
    hotkey:SetFont(fontName, 12/scale, fontFlags);
    
    -- rescale buttonname
    local buttonname = _G[button:GetName().."Name"];
    fontName, fontHeight, fontFlags = buttonname:GetFont();
    buttonname:SetFont(fontName, 10/scale, fontFlags);
end

function CustomButtons_SetButtonAlpha(button, alpha)
    button:SetAlpha(alpha);
	button:SetAttribute("alpha", alpha);
end

function CustomButtons_OnMouseDown(self, button)    
    if (GetCursorInfo()) then
    
        self:Disable();
        self.disable = 1;
        CustomButtons_OnReceiveDrag(self);
        return;
    end
    
    if (button == "RightButton") then
        if (IsControlKeyDown() and InCombatLockdown() == nil) then
            self:Disable();
            self:StartMoving();
        end
    end
    if (button == "LeftButton") then
        if (IsControlKeyDown()) then
            local name = self:GetName();
        
            for index, data in pairs(CustomButtonsData) do
            
                if (data.type == "spell" or data.type == "macro") then
                    local foundname = data.name;
                    
                    if (name == foundname) then                        
                        local group = "";
                        
                        if (data.talent == 0) then
                            group = CB_TALENT_BOTH;
                        elseif (data.talent == 1) then
                            group = CB_TALENT_1;
                        elseif (data.talent == 2) then
                            group = CB_TALENT_2;
                        end
                        
                        DEFAULT_CHAT_FRAME:AddMessage("CustomButtons: "..group, 1.0, 1.0, 1.0, 1, 10);
                    end
                end
            end 
        end
    end
    
    if (CustomButtonsConfig.Sound == 1) then
        if (CustomButtons_CheckUsable(self) == 1) then
            PlaySound("ACTIONBARBUTTONDOWN");
        end
    end
end

function CustomButtons_OnMouseUp(self, button)
    if (self.disable) then
        self:Enable()
        self.disable = nil
    end
                                
    if (button == "RightButton") then
        if (InCombatLockdown() == nil) then
            self:Enable();
            self:StopMovingOrSizing();
            
            CustomButtons_NormalizeButtonPosition(self);
            
            CustomButtons_SaveButton(self);
        end
    end
    
    if (CustomButtonsConfig.Sound == 1) then
        if (CustomButtons_CheckUsable(self) == 1) then
            local sound = self:GetAttribute("sound");
            
            if (sound) then
                PlaySound(sound);
            end
        end
    end
end

function CustomButtons_OnReceiveDrag(self)
    if (InCombatLockdown()) then
        return;
    end
    
    local type, id, itemLink = GetCursorInfo(); 
    
    CustomButtons_OnDragStart(self, "LeftButton");
    
    CustomButtons_UpdateButtonWithActionInfo(self, type, id, itemLink);
    CustomButtons_UpdateUsable(self);
end

function CustomButtons_OnDragStart(self, button)
    if (InCombatLockdown()) then
        return;
    end
    
    if (button == "LeftButton") then
        for i, data in pairs(CustomButtonsData) do
            if (self:GetName() == data.name) then
                if data.type == "item" then
                    PickupItem(data.item);
                elseif data.type == "macro" then
                    PickupMacro(data.macro);
                elseif data.type == "companion" then
                    PickupCompanion(data.ctype, data.index);
                elseif data.type == "equipmentset" then
                    PickupEquipmentSetByName(data.equipmentset);
                elseif data.type == "spell" then
                    PickupSpellBookItem(CustomButtons_findSpellBookId(data.spell, data.spell2, data.id), SpellBookFrame.bookType);
                else
                    ClearCursor();
                end
            end
        end
    else
        ClearCursor();
    end
end

function CustomButtons_NormalizeButtonPosition(self)

    if (not self) then
        return;
    end

    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint(1);
    
    xOfs = ceil(xOfs);
    yOfs = ceil(yOfs);
    
    self:ClearAllPoints();
    self:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs);
end

function CustomButtons_CreateNewButton()

	local i = 1;
	
	local name = "CustomButtonsButton"..i;
	local button = _G[name];
	
	while (button ~= nil) do
		i = i + 1;
	
	    name = "CustomButtonsButton"..i;
	    button = _G[name];
	end
	
	local button = CustomButtons_CreateButton(name);
	
	button:SetScale(1.0);
	button:ClearAllPoints();
	button:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
		
	button:SetScript("OnMouseDown", CustomButtons_OnMouseDown);
	button:SetScript("OnMouseUp", CustomButtons_OnMouseUp);
	
	selectedButton = button;
	
	return button;
end

function CustomButtons_CreateButton(name)
		
	local button = CreateFrame("Button", name, CustomButtonsFrame, "CustomButtonsButtonTemplate");
	
	button:RegisterForDrag("LeftButton","RightButton");
	button:RegisterForClicks("AnyUp");
	button.rangeTimer = -1;
	
 	button:RegisterEvent("PLAYER_TARGET_CHANGED");
 	button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
 	button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
 	
 	button:SetClampedToScreen(true);
		
    local hotkey = _G[name.."HotKey"];
    hotkey:SetText(RANGE_INDICATOR);
    hotkey:Hide();
		
	_G[name.."Cooldown"]:SetFrameStrata("MEDIUM");
		
	if (MASQUE) then
		buttongroup:AddButton(button);
	end
	
	return button;
end

function CustomButtons_DeleteButton()
	local button = selectedButton;
	selectedButton = nil;
	
	if (button) then
		local name = button:GetName();
		CustomButtons_RemoveSavedButton(name);
	
		button:Hide();
		button:SetAttribute("deleted", 1);
		
        if (MASQUE) then
            buttongroup:RemoveButton(button);
        end
	end
end

function CustomButtons_SetButtonSound(sound)
	local button = selectedButton;
	
	if (button) then
		button:SetAttribute("sound", sound);
		CustomButtons_SaveButton(button);
	end
end

function CustomButtons_SetKeyBinding(key, button)
	
	if (button) then	
		local hotkey = _G[button:GetName().."HotKey"];
		
		if (key) then
		    SetOverrideBindingClick(button, false, key, button:GetName());
            CustomButtons_SetKeyBindingText(hotkey, key);
            hotkey:Show();
		else
		
		    local oldkey = button:GetAttribute("key");
		    
		    if (oldkey) then
		        SetOverrideBinding(button, false, oldkey);
		    end
		    
            hotkey:SetText(RANGE_INDICATOR);
            hotkey:Hide();
		end
		
		button:SetAttribute("key", key);
		
		CustomButtons_SaveButton(button);
	end
end

function CustomButtons_ToggleTalent()
        
    local activeSpecGroup = GetActiveSpecGroup();

    if (not activeSpecGroup) then
        activeSpecGroup = 1;
    end
        
	local button = selectedButton;
	
	if (button) then
		local name = button:GetName();

        for index, data in pairs(CustomButtonsData) do
        
            if (data.type == "spell" or data.type == "macro" or data.type == "item") then
                local foundname = data.name;
                
                if (name == foundname) then                    
                    if (data.talent == activeSpecGroup) then
                        data.talent = 0;
                    else
                        data.talent = activeSpecGroup;
                    end
                        
                    local group = "";
                    
                    if (data.talent == 0) then
                        group = CB_TALENT_BOTH;
                    elseif (data.talent == 1) then
                        group = CB_TALENT_1;
                    elseif (data.talent == 2) then
                        group = CB_TALENT_2;
                    end
                    
                    DEFAULT_CHAT_FRAME:AddMessage(group, 1.0, 1.0, 1.0, 1, 10);
                end
            end
        end
	end
end

function CustomButtons_ShowButtonSoundInfo()
 	DEFAULT_CHAT_FRAME:AddMessage("CustomButtons: ".. CB_SOUND_INFO, 1.0, 1.0, 1.0, 1, 10);
end

function CustomButtons_OnUpdate(self, elapsed)

	local rangeTimer = self.rangeTimer;
	if ( rangeTimer ) then
		rangeTimer = rangeTimer - elapsed;

		if ( rangeTimer <= 0 ) then
			local hotkey = _G[self:GetName().."HotKey"];
			
			local valid = -1;
			local type = self:GetAttribute("type");
			if (type == "spell") then
			    local id = CustomButtons_findSpellBookId(self:GetAttribute("cb_sp1"), self:GetAttribute("cb_sp2"), self:GetAttribute("id"));
			    
			    if (id and id >= 0) then  		
                    local isUsable, notEnoughMana = IsUsableSpell(id, SpellBookFrame.bookType);
                    local inRange = IsSpellInRange(id, SpellBookFrame.bookType, "target");	
            
                    if (inRange == 0) then
                        valid = 0;
                    end
                end
  		    elseif (type == "item") then
  		        local item = self:GetAttribute("item");
  		    
  		        local inRange = IsItemInRange(item,"target");
  		        
  		        if (inRange == 0) then
 					valid = 0;
  				end
			end
	
			hotkey:SetVertexColor(1.0, 1.0, 1.0);
			
			if ( hotkey:GetText() == RANGE_INDICATOR ) then
				if ( valid == 0 ) then
					hotkey:Show();
					hotkey:SetVertexColor(1.0, 0.1, 0.1);
				else
					hotkey:Hide();
				end
			else
				if ( valid == 0 ) then
					hotkey:SetVertexColor(1.0, 0.1, 0.1);
				end
			end

            local rangeIndicator = _G[self:GetName().."RangeIndicator"];
            if (not rangeIndicator) then
                rangeIndicator = self:CreateTexture(self:GetName().."RangeIndicator", OVERLAY);
                rangeIndicator:SetAllPoints(self);
                rangeIndicator:SetTexture(1.0, 0.0, 0.0);
                rangeIndicator:SetBlendMode("MOD");
            end
			
			if (CustomButtonsConfig.RangeIndicator == 1) then
                if ( valid == 0 ) then    
                    rangeIndicator:Show();  
                else             
                    rangeIndicator:Hide();  
                end           
            else   
                rangeIndicator:Hide();       
            end
			
			rangeTimer = TOOLTIP_UPDATE_TIME;
		end
		
		self.rangeTimer = rangeTimer;
	end
		
	CustomButtons_UpdateCooldowns(self);
	CustomButtons_UpdateUsable(self);
end

function CustomButtons_ActionButton_OnUpdate(self, elapsed)

    local rangeIndicator = _G[self:GetName().."RangeIndicator"];
    if (not rangeIndicator) then
        rangeIndicator = self:CreateTexture(self:GetName().."RangeIndicator", OVERLAY);
        rangeIndicator:SetAllPoints(self);
        rangeIndicator:SetTexture(1.0, 0.0, 0.0);
        rangeIndicator:SetBlendMode("MOD");
    end

    if (CustomButtonsConfig.RangeIndicator ~= 1) then    
        rangeIndicator:Hide();     
        return;
    end
    
    local spellType, id, subType, spellID = GetActionInfo(self.action);
    
 	if (not spellType) then
        rangeIndicator:Hide(); 
 	else
        local rangeTimer = self.rangeTimer;
        
        if ( rangeTimer ) then
            rangeTimer = rangeTimer - elapsed;
    
            if ( rangeTimer <= 0 ) then
                local valid = IsActionInRange(self.action);
                
                if ( valid == 0 ) then    
                    rangeIndicator:Show();
                else
                    rangeIndicator:Hide(); 
                end
            end
        end
    end
end

function CustomButtons_UpdateCooldowns(self)
		
	local cooldown = _G[self:GetName().."Cooldown"];
	
    local item = self:GetAttribute("item");
    local spell = self:GetAttribute("cb_sp1");
    
    if (item) then
        local id = CustomButtons_GetItemId(item);
        
        if (id) then
            local start, duration, enable = GetItemCooldown(id);
        
            if (cooldown and start and duration and enable) then
                CooldownFrame_SetTimer(cooldown, start, duration, enable);
            end
        end
	elseif (spell) then
	    local id = CustomButtons_findSpellBookId(spell, self:GetAttribute("cb_sp2"), self:GetAttribute("id"));
  		if (id >= 0) then
            local start, duration, enable = GetSpellCooldown(id, SpellBookFrame.bookType);
        
            if (cooldown and start and duration and enable) then
                CooldownFrame_SetTimer(cooldown, start, duration, enable);
            end
        end
	end
end

function CustomButtons_GetItemId(name)
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(name);
    
    if (not itemLink) then
        return nil;
    end
    
    local _, id = strsplit(":", itemLink);

    return id;
end

function CustomButtons_CheckUsable(button)
	local type = button:GetAttribute("type");
  		
	if (type == "spell") then  		
	    local id = CustomButtons_findSpellBookId(button:GetAttribute("cb_sp1"), button:GetAttribute("cb_sp2"), button:GetAttribute("id"));
  		local isUsable, notEnoughMana = IsUsableSpell(id, SpellBookFrame.bookType);
  		
		if ( isUsable ) then
		elseif ( notEnoughMana ) then
			return nil;
		else
			return nil;
		end
	elseif (type == "item") then
  	    local item = button:GetAttribute("item");
  	    
		local count = GetItemCount(item);
		
		if (count == 0) then  
			return nil;	
		end
	end
	
	return 1;
end

function CustomButtons_UpdateUsable(self)

    if (isInConfig) then
        return;
    end;
			
	local icon = _G[self:GetName().."Icon"];
  	local counttext = _G[self:GetName().."Count"];
  	local buttonname = _G[self:GetName().."Name"];
  	
  	buttonname:SetText(""); 

	local type = self:GetAttribute("type");
	local macro = self:GetAttribute("macro");
	local macrotext = self:GetAttribute("macrotext");
	
	if (type == "spell") then
	
  		local spell = self:GetAttribute("cb_sp1");
  		local spell2 = self:GetAttribute("cb_sp2");
	    local id = CustomButtons_findSpellBookId(spell,spell2, self:GetAttribute("id"));
	    
	    if (id and id >= 0) then
  		    local isUsable, notEnoughMana = IsUsableSpell(id, SpellBookFrame.bookType);
            local texture = GetSpellTexture(id, SpellBookFrame.bookType);
            
            if ( notEnoughMana ) then
                icon:SetVertexColor(0.3, 0.3, 0.8);
            elseif (not isUsable ) then
                icon:SetVertexColor(0.3, 0.3, 0.3);
            else
                icon:SetVertexColor(1.0, 1.0, 1.0);
            end
            -- done
            
            if (CustomButtonsConfig.ShowCount == 0 or count == 0) then
                counttext:SetText("");
            else
                if (IsConsumableSpell(id, SpellBookFrame.bookType)) then 
                    local spellCount = GetSpellCount(id, SpellBookFrame.bookType); 
                    
                    if ( isUsable ) and (spellCount == 0) then 
                        -- Some glyph or talent must have changed this to non-consumable spell 
                        counttext:SetText(""); 
                    else 
                        counttext:SetText(spellCount); 
                    end 
                else 
                    countinfo = "";
                    -- check buff
                    local index = 1;
                    local foundname, _, _, foundcount = UnitBuff("player", index);
                    while foundname do 
                        if (foundname == spell) then
                            if (foundcount > 0) then
                                countinfo = foundcount;
                            end
                            break;
                        else
                            index = index + 1;        
                            foundname, _, _, foundcount = UnitBuff("player", index);
                        end
                    end
                    
                    counttext:SetText(countinfo); 
                end 
            end
		end
	elseif (type == "item") then
	
  	    local item = self:GetAttribute("item");
		local count = GetItemCount(item, nil, true); 
	
  		if (count == 0) then  		
			icon:SetVertexColor(0.3, 0.3, 0.3);
  		else    		
			icon:SetVertexColor(1.0, 1.0, 1.0);
  		end
  		
  		if (CustomButtonsConfig.ShowCount == 0 or count == 0) then
    		counttext:SetText("");
  		else
            counttext:SetText(count); 
  		end
		
	elseif (macrotext and string.find(macrotext, "/equipset") == 1) then -- Equipmentset
	
        local name = string.sub(macrotext, 11);
        buttonname:SetText(name);
    	counttext:SetText("");
		
	elseif (macrotext and string.find(macrotext, "/run CallCompanion") == 1) then -- Mount or Pet
	
        buttonname:SetText("");
    	counttext:SetText("");
		
	elseif (macro) then
	
        local name, iconTexture, body = GetMacroInfo(macro); 
        buttonname:SetText(name); 
    	counttext:SetText("");
	
	end
end

function CustomButtons_ResetAttributes(self)
    self:SetAttribute("type", nil);
    self:SetAttribute("item", nil);
    self:SetAttribute("spell", nil);
    --self:SetAttribute("spell2", nil);
    self:SetAttribute("id", nil);
    self:SetAttribute("macro", nil);
    self:SetAttribute("macrotext", nil);
    self:SetAttribute("ctype", nil);
			
    self:SetAttribute("tooltip", nil);
			
    self:SetAttribute("cb_sp1", nil);
    self:SetAttribute("cb_sp2", nil);
end

function CustomButtons_UpdateButtonWithActionInfo(self, type, id, itemLink)
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint(1);
    local texture = nil;
    
    local activeSpecGroup = GetActiveSpecGroup();

    if (not activeSpecGroup) then
        activeSpecGroup = 1;
    end

    if (type == "item") then

        local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(id);
        texture = itemTexture;
        
        CustomButtons_ResetAttributes(self);
        self:SetAttribute("type", "item");
        self:SetAttribute("item", itemName);
        
        CustomButtons_RemoveSavedButton(self:GetName());
        table.insert(CustomButtonsData,
        {
            type = "item";
            name = self:GetName();
            item = itemName;
            texture = itemTexture;
            talent = activeSpecGroup;
            p1 = point;
            p2 = relativePoint;
            x = xOfs;
            y = yOfs;
            scale = 1.0;
            alpha = 1.0;
        } );

    elseif (type == "spell") then
    
        local _, _, offset, numSpells = GetSpellTabInfo(2);

        if (id <= offset+numSpells) then    
            local name, subname, itemTexture = GetSpellInfo(id, SpellBookFrame.bookType);
            texture = itemTexture;
            
            local spellLink, tradeSkillLink = GetSpellLink(id, SpellBookFrame.bookType);
    
            CustomButtons_ResetAttributes(self);
            self:SetAttribute("type", "spell");
            
            local spellname = name;
            if (subname and strlen(subname) > 0) then
                spellname = name.."("..subname..")";
            end
            
            self:SetAttribute("spell", spellname);            
            
            self:SetAttribute("cb_sp1", name);
            self:SetAttribute("cb_sp2", subname);
                
            self:SetAttribute("id", id);
            
            CustomButtons_RemoveSavedButton(self:GetName());
            table.insert(CustomButtonsData,
            {
                type = "spell";
                name = self:GetName();
                spell = name;
                spell2 = subname;
                id = id;
                texture = itemTexture;
                talent = activeSpecGroup;
                p1 = point;
                p2 = relativePoint;
                x = xOfs;
                y = yOfs;
                scale = 1.0;
                alpha = 1.0;
            } );
        end

    elseif (type == "macro") then
    
        local name, itemTexture, body, _ = GetMacroInfo(id);		
        texture = itemTexture;

        CustomButtons_ResetAttributes(self);
        self:SetAttribute("type", "macro");
        self:SetAttribute("macro", name);
        
        CustomButtons_RemoveSavedButton(self:GetName());
        table.insert(CustomButtonsData,
        {
            type = "macro";
            name = self:GetName();
            macro = name;
            texture = itemTexture;
            talent = activeSpecGroup;
            p1 = point;
            p2 = relativePoint;
            x = xOfs;
            y = yOfs;
            scale = 1.0;
            alpha = 1.0;
        } );
        
        self:SetAttribute("tooltip", name);

    elseif (type == "equipmentset") then
    
        local itemTexture, lessIndex = GetEquipmentSetInfoByName(id);
        itemTexture = "Interface\\Icons\\"..itemTexture;		
        texture = itemTexture;
        
        CustomButtons_ResetAttributes(self);
        self:SetAttribute("type", "macro");
        self:SetAttribute("macrotext", "/equipset "..id);
        
        self:SetAttribute("tooltip", id);
        
        CustomButtons_RemoveSavedButton(self:GetName());
        table.insert(CustomButtonsData,
        {
            type = "equipmentset";
            name = self:GetName();
            equipmentset = id;
            texture = itemTexture;
            p1 = point;
            p2 = relativePoint;
            x = xOfs;
            y = yOfs;
            scale = 1.0;
            alpha = 1.0;
        } );

    elseif (type == "companion") then
        local creatureID, creatureName, creatureSpellID, itemTexture, issummoned = GetCompanionInfo(itemLink, id);
        texture = itemTexture;
        
        CustomButtons_ResetAttributes(self);
        self:SetAttribute("type", "macro");
        
        if (itemLink == "CRITTER") then
            self:SetAttribute("macrotext", "/run CallCompanion(\""..itemLink.."\", "..id..")");
        else
            self:SetAttribute("macrotext", "/run if IsMounted() then Dismount() else CallCompanion(\""..itemLink.."\", "..id..") end");
        end
        
        self:SetAttribute("tooltip", id);
    
        CustomButtons_RemoveSavedButton(self:GetName());
        table.insert(CustomButtonsData,
        {
            type = "companion";
            name = self:GetName();
            ctype = itemLink;
            index = id;
            texture = itemTexture;
            p1 = point;
            p2 = relativePoint;
            x = xOfs;
            y = yOfs;
            scale = 1.0;
            alpha = 1.0;
        } );
        
    end
            
    local icon = _G[self:GetName().."Icon"];
    icon:SetTexture(texture);
    
    CustomButtons_SaveButton(self);
    CustomButtons_UpdateUsable(self);
end

function CustomButtons_GetSpellID(name)

    local foundname;

    for i = 1, 1000000 do
        local foundname = GetSpellInfo(i);
        if name == foundname then
            return i;
        end
    end
end

function CustomButtons_SaveButton(button)

    local type = button:GetAttribute("type");
    
    if (type == nil) then
        return;
    end

	local name = button:GetName();
    
    for index, data in pairs(CustomButtonsData) do
		local foundname = data.name;
		
		if (name == foundname) then
			
			point, relativeTo, relativePoint, xOfs, yOfs = button:GetPoint(1);
			
		    data.p1 = point;
		    data.p2 = relativePoint;
		    data.x = xOfs;
		    data.y = yOfs;
		    data.scale = button:GetScale();
		    data.alpha = button:GetAlpha();
		    
		    data.sound = button:GetAttribute("sound");
		    data.key = button:GetAttribute("key");
		end
		
    end
end

function CustomButtons_RemoveSavedButton(name)
	for index, data in pairs(CustomButtonsData) do
		local foundname = data.name;
		
		if (name == foundname) then
		    table.remove(CustomButtonsData, index);
			return;
		end
    end
end

function CustomButtons_MouseOver(button)
	CustomButtons_SetTooltip(button);
	selectedButton = button;
end

function CustomButtons_MouseOut(button)
	GameTooltip:FadeOut();
	selectedButton = nil;
end

function CustomButtons_findSpellBookId(spell, spell2, id)   
	id = id or -1;

    if (GetSpellInfo(spell)) then
       
        local _, _, offset, numSpells = GetSpellTabInfo(2);
    
        for i = 1, offset + numSpells do
            local spellName, spellSubName = GetSpellBookItemName(i, BOOKTYPE_SPELL);
            
            if (not spellName) then
                return id;
            end
            
            if (spell2 and spellName == spell and spellSubName == spell2) then
                return i;
            elseif ((not spell2) and spellName == spell) then
                return i;
            end
        end
    end
      
    return id;
end

function CustomButtons_SetTooltip(button)

	local type = button:GetAttribute("type");
	local macro = button:GetAttribute("macro");
	local macrotext = button:GetAttribute("macrotext");
	
  	local item = button:GetAttribute("item");
    local spell = button:GetAttribute("cb_sp1");
    local spell2 = button:GetAttribute("cb_sp2");
    local id = button:GetAttribute("id");
	
	local tooltip = button:GetAttribute("tooltip");
	
	CustomButtons_ShowTooltip(button, type, item, spell, spell2, id, macro, macrotext, tooltip);
end

function CustomButtons_ShowTooltip(button, type, item, spell, spell2, id, macro, macrotext, tooltip)
  	
	if (type == "item") then	
        local count = GetItemCount(item); 
        local itemName, itemLink = GetItemInfo(item); 
        
        -- Look for inventory item that have a "use", update the itemLink if found 
        InventorySlotNames = { "HeadSlot", "NeckSlot", "ShoulderSlot",
            "BackSlot", "ChestSlot", "ShirtSlot", "TabardSlot", "WristSlot", 
            "HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot", "Finger0Slot", 
            "Finger1Slot", "Trinket0Slot", "Trinket1Slot", "MainHandSlot", 
            "SecondaryHandSlot"}; 
        
        for index, slotName in pairs(InventorySlotNames) do 
            local slotId, textureName = GetInventorySlotInfo(slotName); 
            local itemLinkCheck = GetInventoryItemLink("player", slotId);
            
            if (itemName and itemLinkCheck) then 
                local found = string.find(itemLinkCheck, itemName); 
                
                if (found) then 
                    itemLink = itemLinkCheck; 
                end 
            end 
        end 
        
        GameTooltip_SetDefaultAnchor(GameTooltip, button); 
        
        if (itemLink) then 
            GameTooltip:SetHyperlink(itemLink); 
        end 
        
    elseif (type == "spell") then
	    
	    local id = CustomButtons_findSpellBookId(spell, spell2, id);
	    
	    if (id and id > 0) then
    		GameTooltip_SetDefaultAnchor(GameTooltip, button);	
        	GameTooltip:SetSpellBookItem(id, SpellBookFrame.bookType);
		end
		
	elseif (macrotext and string.find(macrotext, "/equipset") == 1) then -- Equipmentset
	    
		GameTooltip_SetDefaultAnchor(GameTooltip, button);
        GameTooltip:SetText(tooltip);
		
	elseif (macrotext and string.find(macrotext, "/run CallCompanion") == 1) then -- Pet
	    
	    local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("CRITTER", tooltip);
		GameTooltip_SetDefaultAnchor(GameTooltip, button);
        GameTooltip:SetHyperlink("spell:"..creatureSpellID);
		
	elseif (macrotext and string.find(macrotext, "/run if IsMounted") == 1) then -- Mount
	    
	    local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", tooltip);
		GameTooltip_SetDefaultAnchor(GameTooltip, button);
        GameTooltip:SetHyperlink("spell:"..creatureSpellID);
        
	elseif (macro) then
	    
		GameTooltip_SetDefaultAnchor(GameTooltip, button);
        GameTooltip:SetText(tooltip);
 	    
	end
        
    GameTooltip:AppendText("");
end