local buttons_loaded = false;

local selectedIndex = nil;
local custombuttons = {};

-- functions
function CustomButtonsConfig_OnLoad(self)
 	DEFAULT_CHAT_FRAME:AddMessage("|c333399ffNirriti's |rCustomButtons Config loaded.", 1.0, 1.0, 1.0, 1, 10);
 	
 	-- Hauptfenster
	self.name="CustomButtons";
    self.addonname = "CustomButtons";
    InterfaceOptions_AddCategory(self);
    
    hooksecurefunc(InterfaceOptionsFrame, "Show", CustomButtonsConfig_InterfaceOptionsFrame_Show);
    hooksecurefunc(InterfaceOptionsFrame, "Hide", CustomButtonsConfig_InterfaceOptionsFrame_Hide);
end

function CustomButtonsConfig_InterfaceOptionsFrame_Show()
    CustomButtonsConfig_Init();
    isInConfig = true;
end

function CustomButtonsConfig_InterfaceOptionsFrame_Hide()
    CustomButtonConfigEntry_DeColorizeAllButtons();
    isInConfig = false;
end

function CustomButtonConfigEntry_OnClick(self, button, down)
    PlaySound("ACTIONBARBUTTONDOWN");
    selectedIndex = self.index;
    
    local data = custombuttons[self.index];
    
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	dataFrame.data = data;
	
	if (not dataFrame:IsShown()) then
	    dataFrame:Show();
	end
	
	CustomButtonConfigEntry_UpdateButtonData();
			
	CustomButtonConfigEntry_ColorizeButton(data.name);
end
 
function CustomButtonsConfig_UpdateButton()       
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	data = dataFrame.data;
	
	if (not data) then
	    return;
	end
	
	local buttonname = data.name;	
	local button = _G[buttonname];
    
    CustomButtons_ReinitSpells();
end
 
function CustomButtonsConfig_ResetSelection()   

	local dataFrame = _G["CustomButtonConfig_ButtonData"];
    
    -- reset selection
    selectedIndex = nil;
    CustomButtonsConfig_OnUpdate();
	dataFrame:Hide();
	
    CustomButtonConfigEntry_UpdateButtonData();
    
    CustomButtons_ReinitSpells();
    CustomButtonsConfig_Init();
    
    CustomButtonConfigEntry_DeColorizeAllButtons();
end

function CustomButtonConfigEntry_DeColorizeAllButtons()    
	for index, data in pairs(CustomButtonsData) do
        local icon = _G[data.name.."Icon"];
        if (icon) then
            icon:SetVertexColor(1.0, 1.0, 1.0);
        end
	end
end

function CustomButtonConfigEntry_ColorizeButton(name)  
    CustomButtonConfigEntry_DeColorizeAllButtons();
	local icon = _G[name.."Icon"];
	if (icon) then
        icon:SetVertexColor(0.0, 1.0, 0.0);
	end
end

function CustomButtonConfigEntry_UpdateButtons()    
    CustomButtons_ReinitSpells();
end

function CustomButtonConfigEntry_OnEnter(self)    
    local data = custombuttons[self.index];    
    --CustomButtons_ShowTooltip(self, data.type, data.item, data.spell, data.spell2, data.macro, data.macrotext, data.tooltip);
end

function CustomButtonConfigEntry_OnLeave(self)
	--GameTooltip:FadeOut();
end

function CustomButtonsConfig_Sort(a, b)
    if (a.type < b.type) then
        return true;
    end
    
    if (a.type == b.type) then
		if (a.type == "item") then	
		    return a.item < b.item;
		elseif (a.type == "spell") then
		    return a.spell < b.spell;
		elseif (a.type == "macro") then
		    return a.macro < b.macro;
		elseif (a.type == "equipmentset") then
		    return a.equipmentset < b.equipmentset;
		elseif (a.type == "companion") then
		    return a.index < b.index;
		end
    end
    
    return false;
end

function CustomButtonsConfig_Init()
    custombuttons = {};
    
	for index, data in pairs(CustomButtonsData) do
	    if (data.type) then
            tinsert(custombuttons, data);
        end
	end
    
	table.sort(custombuttons, function(a,b) return CustomButtonsConfig_Sort(a, b) end);

    -- create gui
	HybridScrollFrame_OnLoad(CustomButtonsConfigFrame.scrollFrame);
	CustomButtonsConfigFrame.scrollFrame.update = CustomButtonsConfig_OnUpdate;
	HybridScrollFrame_CreateButtons(CustomButtonsConfigFrame.scrollFrame, "CustomButtonConfigEntryTemplate");
	
    CustomButtonsConfigFrame.scrollFrame:SetFrameStrata("HIGH");
	
	local borderframe = _G["CustomButtonsConfigFrameBorderFrame"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", "CustomButtonsConfigFrameBorderFrame", CustomButtonsConfigFrame.scrollFrame);
	end
	borderframe:SetPoint("TOPLEFT", -10, 5);
	borderframe:SetPoint("BOTTOMRIGHT", 25, -5);
    borderframe:SetFrameStrata("MEDIUM");

	borderframe:SetBackdrop( { 
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    });
    borderframe:SetBackdropColor(0.0, 0.0, 0.0, 1.0);	

	CustomButtonConfig_ButtonData:SetBackdrop( { 
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    });
    CustomButtonConfig_ButtonData:SetBackdropColor(0.0, 0.0, 0.0, 1.0);	
    
    CustomButtonsConfig_OnUpdate();
    
    CustomButtonsConfigFrameToggleCount:SetChecked(CustomButtonsConfig.ShowCount);
    CustomButtonsConfigFrameToggleSound:SetChecked(CustomButtonsConfig.Sound);
    CustomButtonsConfigFrameToggleRange:SetChecked(CustomButtonsConfig.RangeIndicator);

    buttons_loaded = true;
end

function CustomButtonsConfig_OnUpdate()	
    if (not buttons_loaded) then
        return;
    end
	
	CustomButtonConfigEntryHighlightFrame:Hide();
	
	local buttons = CustomButtonsConfigFrame.scrollFrame.buttons;
	
	local numEntries = #custombuttons;
	local numButtons = #buttons;
		    
	local scrollOffset = HybridScrollFrame_GetOffset(CustomButtonsConfigFrame.scrollFrame);
	local buttonHeight = buttons[1]:GetHeight();
	local displayedHeight = 0;
	
	local entry;
	local data;
	
	for i=1, numButtons do
	
		local entryIndex = i + scrollOffset;
		entry = buttons[i];
		
		if ( entryIndex <= numEntries ) then
		    data = custombuttons[entryIndex];
		    
			entry:Show();
		    entry.index = entryIndex;
		    
		    -- button data		    
            if (data.type == "spell") then
		        entry:SetText(data.spell);
		        entry.type:SetText(SPELLS);
            elseif (data.type == "item") then
		        entry:SetText(data.item);
		        entry.type:SetText(HELPFRAME_ITEM_TITLE);
            elseif (data.type == "macro") then
		        entry:SetText(data.macro);
		        entry.type:SetText(MACRO);
            elseif (data.type == "equipmentset") then
		        entry:SetText(data.equipmentset);
		        entry.type:SetText("EquipmentSet");
            elseif (data.type == "companion") then
                local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo(data.ctype, data.index);
		        entry:SetText(creatureName);
		        if (data.ctype == "CRITTER") then
		            entry.type:SetText(COMPANIONS);
		        else
		            entry.type:SetText(MOUNTS);
		        end
            end
            
            entry.texture:SetTexture(data.texture);
            
            if (GetNumSpecGroups() > 1) then            
                if (data.talent == 1 or data.talent == 2) then       
                    entry.talent:SetText(CustomButtonsConfig_DetectTalentName(data.talent));
                else
                    entry.talent:SetText("");
                end
            else
                entry.talent:SetText("");
		    end
		    
            -- this isn't a header, hide the header textures
            entry:SetNormalTexture("");
            entry:SetHighlightTexture("");
            
            if (entryIndex == selectedIndex) then
                -- reposition highlight frames
                CustomButtonConfigEntryHighlightFrame:SetParent(entry);
                CustomButtonConfigEntryHighlightFrame:SetPoint("TOPLEFT", entry, "TOPLEFT", 0, 0);
                CustomButtonConfigEntryHighlightFrame:SetPoint("BOTTOMRIGHT", entry, "BOTTOMRIGHT", 0, 0);
		        CustomButtonConfigEntryHighlightFrame:Show();
            end
		else
			entry:Hide();
		end
		
		displayedHeight = displayedHeight + buttonHeight;
	end
	
	HybridScrollFrame_Update(CustomButtonsConfigFrame.scrollFrame, numEntries * buttonHeight, displayedHeight);
end

function CustomButtonsConfig_DetectTalentName(index)	
    local primaryTree = GetSpecialization(false, false, index);
    
    if (primaryTree) then
        local id, name, description, icon, background, role = GetSpecializationInfo(primaryTree);  
        return name;
    end
    
    return "";
end

function CustomButtonConfigEntry_UpdateButtonData()    
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
	
	if (not data) then
	    dataFrame.name:SetText("");
        dataFrame.type:SetText("");
        dataFrame.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
        
        return;
	end
    
    local buttonname = data.name;   
    local button = _G[buttonname];
	
    if (data.type == "spell") then
        dataFrame.name:SetText(data.spell);
        dataFrame.type:SetText(SPELLS);
    elseif (data.type == "item") then
        dataFrame.name:SetText(data.item);
        dataFrame.type:SetText(HELPFRAME_ITEM_TITLE);
    elseif (data.type == "macro") then
        dataFrame.name:SetText(data.macro);
        dataFrame.type:SetText(MACRO);
    elseif (data.type == "equipmentset") then
        dataFrame.name:SetText(data.equipmentset);
        dataFrame.type:SetText("EquipmentSet");
    elseif (data.type == "companion") then
        local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo(data.ctype, data.index);
        dataFrame.name:SetText(creatureName);
        if (data.ctype == "CRITTER") then
            dataFrame.type:SetText(COMPANIONS);
        else
            dataFrame.type:SetText(MOUNTS);
        end
    end
    
    UIDropDownMenu_ClearAll(dataFrame.talent_dropdown);
    UIDropDownMenu_Initialize(dataFrame.talent_dropdown, CustomButtonConfig_TalentDropDown_Initialize);
    
    if (data.type == "spell" or data.type == "macro" or data.type == "item") then
        dataFrame.talent_dropdown:Show();
    else
        dataFrame.talent_dropdown:Hide();
    end
    
    if (GetNumSpecGroups() > 1) then
        if (data.talent == 1 or data.talent == 2) then           
            UIDropDownMenu_SetSelectedValue(dataFrame.talent_dropdown, data.talent);
            UIDropDownMenu_SetText(dataFrame.talent_dropdown, CustomButtonsConfig_DetectTalentName(data.talent));
        else    
            UIDropDownMenu_SetSelectedValue(dataFrame.talent_dropdown, 3);
            UIDropDownMenu_SetText(dataFrame.talent_dropdown, ALL);
        end
    else    
        UIDropDownMenu_SetSelectedValue(dataFrame.talent_dropdown, 1);
        UIDropDownMenu_SetText(dataFrame.talent_dropdown, ALL);
    end
    
    dataFrame.scale:SetValue(data.scale);
    _G[dataFrame.scale:GetName().."Value"]:SetText(string.format("%.2f", data.scale));
    
    dataFrame.alpha:SetValue(data.alpha);
    _G[dataFrame.alpha:GetName().."Value"]:SetText(string.format("%.2f", data.alpha));
    
    dataFrame.texture:SetTexture(data.texture);
    
    if (data.sound) then
        dataFrame.sound:SetText(data.sound);
	else
        dataFrame.sound:SetText("");
	end
	
    local keybinding = button:GetAttribute("key");
    if (keybinding) then
        dataFrame.keybinding:SetText(keybinding);
	else
        dataFrame.keybinding:SetText("");
	end
	
	CustomButtons_NormalizeButtonPosition(button);
end
 
function CustomButtonsConfig_NewButtonClicked()  

	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	dataFrame.data = nil; 

    local button = CustomButtons_CreateNewButton();
    CustomButtons_UpdateButtonWithActionInfo(button, "spell", 1, nil);
        
    CustomButtonsConfig_ResetSelection();
end
 
function CustomButtonsConfig_DeleteButtonClicked()       
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
	
	if (not data) then
	    return;
	end
	
	local buttonname = data.name;	
	local button = _G[buttonname];
	
	CustomButtons_RemoveSavedButton(buttonname);
	
	button:Hide();
	button:SetAttribute("deleted", 1);
        
    CustomButtonsConfig_ResetSelection();
end
 
function CustomButtonConfig_SelectScale(self)
    local value = self:GetValue();
    _G[self:GetName().."Value"]:SetText(string.format("%.2f", value));
    
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
	
	if (data) then
        data.scale = value;
        
        local buttonname = data.name;	
        local button = _G[buttonname];
        
        CustomButtons_SetButtonScale(button, value);
    end
end
 
function CustomButtonConfig_SelectAlpha(self)
    local value = self:GetValue();
    _G[self:GetName().."Value"]:SetText(string.format("%.2f", value));
    
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
		
	if (data) then
        data.alpha = value;
        
        local buttonname = data.name;	
        local button = _G[buttonname];
        
        CustomButtons_SetButtonAlpha(button, value);
    end
end
 
function CustomButtonConfig_SelectPosX(self)
    local value = self:GetValue();
    self:SetValue(0);
    
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
        
    if (not data) then
        return;
    end
    
    local buttonname = data.name;	
    local button = _G[buttonname];
	
    local point, relativeTo, relativePoint, xOfs, yOfs = button:GetPoint(1);
    xOfs = xOfs + value;
    button:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs);
    CustomButtons_SaveButton(button);
end

function CustomButtonConfig_SelectPosY(self)
    local value = self:GetValue();
    self:SetValue(0);
        
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
        
    if (not data) then
        return;
    end
	
    local buttonname = data.name;	
    local button = _G[buttonname];
	
    local point, relativeTo, relativePoint, xOfs, yOfs = button:GetPoint(1);
    yOfs = yOfs + value;
    button:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs);
    CustomButtons_SaveButton(button);
end

function CustomButtonConfig_ChangePosition(direction, value)	
        
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
        
    if (not data) then
        return;
    end
	
    local buttonname = data.name;	
    local button = _G[buttonname];
    
    local point, relativeTo, relativePoint, xOfs, yOfs = button:GetPoint(1);
    
    if (direction == "x") then
        xOfs = xOfs + value;
    elseif (direction == "y") then
        yOfs = yOfs + value;
    end
    
    button:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs);
    CustomButtons_SaveButton(button);
end

function CustomButtonConfig_SelectTalent(index)
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
	
    data.talent = index;
    
    CustomButtonConfigEntry_UpdateButtonData();
    CustomButtonsConfig_UpdateButton();
end

function CustomButtonConfig_TalentDropDown_Initialize(self)
    if (GetNumSpecGroups() > 1) then
        for i = 1, GetNumSpecGroups() do
            local info = UIDropDownMenu_CreateInfo();
            info.text = CustomButtonsConfig_DetectTalentName(i);
            info.value = i;
            info.func = function()
                    CustomButtonConfig_SelectTalent(i);
                end
            UIDropDownMenu_AddButton(info);
        end
    end
        
    local info = UIDropDownMenu_CreateInfo();
    info.text = ALL;
    info.value = i;
    info.func = function()
            CustomButtonConfig_SelectTalent(0);
        end
    UIDropDownMenu_AddButton(info);
end

function CustomButtonConfig_ChangeSound(self) 
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
	
    local newSound = self:GetText();
    if (newSound == "") then
        newSound = nil;
    end
    
    data.sound = newSound;
end

function CustomButtonConfig_PlaySound() 
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
	
    if (data.sound) then
        PlaySound(data.sound);
    end	
end

function CustomButtonConfigSound_OnEnter(self)    
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    GameTooltip:SetText(ENABLE_SOUNDFX);      
    GameTooltip:AddLine(CB_SOUND_INFO);        
    GameTooltip:AppendText("");
    GameTooltip:Show();
end

function CustomButtonConfigSound_OnLeave(self)
	GameTooltip:FadeOut();
end

function CustomButtonConfig_SaveKeyBinding(self) 
	local dataFrame = _G["CustomButtonConfig_ButtonData"];
	local data = dataFrame.data;
	
	local buttonname = data.name;	
	local button = _G[buttonname];
	
    local newKeyBinding = dataFrame.keybinding:GetText();
    
    CustomButtons_SetKeyBinding(newKeyBinding, button);
end

function CustomButtonConfigKeyBinding_OnEnter(self)    
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    GameTooltip:SetText(KEY_BINDING);      
    GameTooltip:AddLine("");        
    GameTooltip:AppendText("");
    GameTooltip:Show();
end

function CustomButtonConfigKeyBinding_OnLeave(self)
	GameTooltip:FadeOut();
end